const {
  time,
  loadFixture,
} = require('@nomicfoundation/hardhat-toolbox/network-helpers');
const { expect } = require('chai');
const { ethers } = require('hardhat');

describe('Work1', () => {
  const WORK1_PRICE = 100;
  async function deploy() {
    // Contracts are deployed using the first signer/account by default
    const [account1, account2] = await ethers.getSigners();

    const DAO = await ethers.getContractFactory('DAO');
    const dao = await DAO.deploy();
    const Work1 = await ethers.getContractFactory('Work1');
    const work1 = await Work1.deploy(dao.target, WORK1_PRICE);

    await dao.initiateValidationPool(account1, 0, { value: 100 });
    await dao.evaluateOutcome(0);

    return {
      dao, work1, account1, account2,
    };
  }

  it('Should deploy', async () => {
    const { dao, work1, account1 } = await loadFixture(deploy);
    expect(dao).to.exist;
    expect(work1).to.exist;
    expect(await dao.memberCount()).to.equal(1);
    expect(await dao.balanceOf(account1)).to.equal(100);
    expect(await dao.totalSupply()).to.equal(100);
    expect(await work1.stakeCount()).to.equal(0);
  });

  it('Should be able to receive availability stake', async () => {
    const { dao, work1, account1 } = await loadFixture(deploy);
    await dao.stakeAvailability(work1.target, 50, 60);
    expect(await dao.balanceOf(account1)).to.equal(50);
    expect(await dao.balanceOf(work1.target)).to.equal(50);
    expect(await work1.stakeCount()).to.equal(1);
    const stake = await work1.stakes(0);
    expect(stake.worker).to.equal(account1);
    expect(stake.amount).to.equal(50);
    expect(stake.endTime).to.equal(await time.latest() + 60);
  });
});
